@@ -12,9 +12,8 @@
 #include <olm/pk.h>
 
 #include <algorithm>
-#include <iomanip>
-#include <iostream>
 
+#include "mtx/log.hpp"
 #include "mtxclient/crypto/client.hpp"
 
 namespace mtx {
@@ -68,7 +67,9 @@ key_from_passphrase(const std::string &password,
       std::string(32, '\0'), testKeys.aes, to_binary_buf(base642bin(parameters.iv)));
 
     auto mac = HMAC_SHA256(testKeys.mac, encrypted);
-    if (bin2base64(to_string(mac)) != parameters.mac) {
+    if (mac != to_binary_buf(base642bin(parameters.mac))) {
+        mtx::utils::log::log()->debug(
+          "mac mismatch: {} != {}", bin2base64(to_string(mac)), parameters.mac);
         return std::nullopt;
     }
 
@@ -101,7 +102,9 @@ key_from_recoverykey(const std::string &recoverykey,
       std::string(32, '\0'), testKeys.aes, to_binary_buf(base642bin(parameters.iv)));
 
     auto mac = HMAC_SHA256(testKeys.mac, encrypted);
-    if (bin2base64(to_string(mac)) != parameters.mac) {
+    if (mac != to_binary_buf(base642bin(parameters.mac))) {
+        mtx::utils::log::log()->debug(
+          "mac mismatch: {} != {}", bin2base64(to_string(mac)), parameters.mac);
         return std::nullopt;
     }
 
@@ -132,7 +135,9 @@ decrypt(const mtx::secret_storage::AesHmacSha2EncryptedData &data,
     auto keys   = HKDF_SHA256(decryptionKey, BinaryBuf(32, 0), to_binary_buf(key_name));
     auto keyMac = HMAC_SHA256(keys.mac, to_binary_buf(base642bin(data.ciphertext)));
 
-    if (bin2base64(to_string(keyMac)) != data.mac) {
+    if (keyMac != to_binary_buf(base642bin(data.mac))) {
+        mtx::utils::log::log()->debug(
+          "mac mismatch: {} != {}", bin2base64(to_string(keyMac)), data.mac);
         return "";
     }
 
@@ -411,9 +416,9 @@ mtx::responses::backup::SessionData
 decrypt_session(const mtx::responses::backup::EncryptedSessionData &data,
                 const BinaryBuf &privateKey)
 {
-    mtx::responses::backup::SessionData decrypted = json::parse(
-      CURVE25519_AES_SHA2_Decrypt(data.ciphertext, privateKey, data.ephemeral, data.mac));
-    return decrypted;
+    return nlohmann::json::parse(
+             CURVE25519_AES_SHA2_Decrypt(data.ciphertext, privateKey, data.ephemeral, data.mac))
+      .get<mtx::responses::backup::SessionData>();
 }
 
 std::string
@@ -545,15 +550,6 @@ HMAC_SHA256(const BinaryBuf &hmacKey, const BinaryBuf &data)
     return output;
 }
 
-void
-print_binary_buf(const BinaryBuf &buf)
-{
-    for (uint8_t val : buf) {
-        std::cout << std::to_string(val) << ",";
-    }
-    std::cout << std::endl;
-}
-
 void
 uint8_to_uint32(uint8_t b[4], uint32_t &u32)
 {
