@@ -55,7 +55,7 @@ coeurl::Headers
 mtx::http::Client::prepare_headers(bool requires_auth)
 {
     coeurl::Headers headers;
-    headers["User-Agent"] = "mtxclient v0.8.0";
+    headers["User-Agent"] = "mtxclient v0.8.2";
 
     if (requires_auth && !access_token_.empty())
         headers["Authorization"] = "Bearer " + access_token();
@@ -684,6 +684,16 @@ Client::versions(Callback<mtx::responses::Versions> callback)
         const mtx::responses::Versions &res, HeaderFields, RequestErr err) { callback(res, err); });
 }
 
+void
+Client::capabilities(Callback<mtx::responses::capabilities::Capabilities> callback)
+{
+    get<mtx::responses::capabilities::Capabilities>(
+      "/client/v3/capabilities",
+      [callback = std::move(callback)](const mtx::responses::capabilities::Capabilities &res,
+                                       HeaderFields,
+                                       RequestErr err) { callback(res, err); });
+}
+
 void
 Client::upload(const std::string &data,
                const std::string &content_type,
@@ -1235,6 +1245,58 @@ Client::get_hierarchy(const std::string &room_id,
         const mtx::responses::HierarchyRooms &res, HeaderFields, RequestErr err) { cb(res, err); });
 }
 
+void
+Client::get_summary(const std::string &room_id,
+                    Callback<mtx::responses::PublicRoomsChunk> cb,
+                    std::vector<std::string> via)
+{
+    std::string query;
+    if (!via.empty()) {
+        query = "?via=" + mtx::client::utils::url_encode(via[0]);
+        for (size_t i = 1; i < via.size(); i++) {
+            query += "&via=" + mtx::client::utils::url_encode(via[i]);
+        }
+    }
+    std::string api_path = "/client/unstable/im.nheko.summary/rooms/" +
+                           mtx::client::utils::url_encode(room_id) + "/summary" + query;
+
+    get<mtx::responses::PublicRoomsChunk>(
+      api_path,
+      [this, room_id, cb = std::move(cb)](
+        const mtx::responses::PublicRoomsChunk &res, HeaderFields, RequestErr err) {
+          if (!err || !(err->status_code == 404 || err->status_code == 400))
+              cb(res, err);
+          else if (!room_id.empty() && room_id[0] == '#')
+              resolve_room_alias(
+                room_id, [this, cb](const mtx::responses::RoomId &room, RequestErr err) {
+                    if (room.room_id.empty())
+                        cb({}, err);
+                    else
+                        get_hierarchy(
+                          room.room_id,
+                          [cb](const mtx::responses::HierarchyRooms &res, RequestErr err) {
+                              if (res.rooms.empty())
+                                  cb({}, err);
+                              else
+                                  cb(res.rooms.front(), err);
+                          },
+                          "",
+                          1);
+                });
+          else
+              get_hierarchy(
+                room_id,
+                [cb](const mtx::responses::HierarchyRooms &res, RequestErr err) {
+                    if (res.rooms.empty())
+                        cb({}, err);
+                    else
+                        cb(res.rooms.front(), err);
+                },
+                "",
+                1);
+      });
+}
+
 //
 // Device related endpoints
 //
