@@ -44,6 +44,8 @@ to_string(ErrorCode code)
         return "M_MISSING_TOKEN";
     case ErrorCode::M_INVALID_SIGNATURE:
         return "M_INVALID_SIGNATURE";
+    case ErrorCode::M_EXCLUSIVE:
+        return "M_EXCLUSIVE";
     case ErrorCode::M_UNKNOWN:
         return "M_UNKNOWN";
     }
@@ -54,9 +56,7 @@ to_string(ErrorCode code)
 ErrorCode
 from_string(const std::string &code)
 {
-    if (code == "M_UNRECOGNIZED")
-        return ErrorCode::M_UNRECOGNIZED;
-    else if (code == "M_FORBIDDEN")
+    if (code == "M_FORBIDDEN")
         return ErrorCode::M_FORBIDDEN;
     else if (code == "M_UNKNOWN_TOKEN")
         return ErrorCode::M_UNKNOWN_TOKEN;
@@ -88,9 +88,11 @@ from_string(const std::string &code)
         return ErrorCode::M_MISSING_TOKEN;
     else if (code == "M_INVALID_SIGNATURE")
         return ErrorCode::M_INVALID_SIGNATURE;
+    else if (code == "M_EXCLUSIVE")
+        return ErrorCode::M_EXCLUSIVE;
     else if (code == "M_UNKNOWN")
         return ErrorCode::M_UNKNOWN;
-    else
+    else // if (code == "M_UNRECOGNIZED")
         return ErrorCode::M_UNRECOGNIZED;
 }
 
