@@ -77,6 +77,13 @@ getEventType(const std::string &type)
     else if (type == "m.sticker")
         return EventType::Sticker;
 
+    else if (type == "m.policy.rule.user")
+        return EventType::PolicyRuleUser;
+    else if (type == "m.policy.rule.room")
+        return EventType::PolicyRuleRoom;
+    else if (type == "m.policy.rule.server")
+        return EventType::PolicyRuleServer;
+
     else if (type == "m.space.child")
         return EventType::SpaceChild;
     else if (type == "m.space.parent")
@@ -98,6 +105,12 @@ getEventType(const std::string &type)
         return EventType::CallAnswer;
     else if (type == "m.call.hangup")
         return EventType::CallHangUp;
+    else if (type == "m.call.select_answer")
+        return EventType::CallSelectAnswer;
+    else if (type == "m.call.reject")
+        return EventType::CallReject;
+    else if (type == "m.call.negotiate")
+        return EventType::CallNegotiate;
     else if (type == "m.secret.request")
         return EventType::SecretRequest;
     else if (type == "m.secret.send")
@@ -191,6 +204,13 @@ to_string(EventType type)
     case EventType::Sticker:
         return "m.sticker";
 
+    case EventType::PolicyRuleUser:
+        return "m.policy.rule.user";
+    case EventType::PolicyRuleRoom:
+        return "m.policy.rule.room";
+    case EventType::PolicyRuleServer:
+        return "m.policy.rule.server";
+
     case EventType::SpaceChild:
         return "m.space.child";
     case EventType::SpaceParent:
@@ -212,6 +232,12 @@ to_string(EventType type)
         return "m.call.answer";
     case EventType::CallHangUp:
         return "m.call.hangup";
+    case EventType::CallSelectAnswer:
+        return "m.call.select_answer";
+    case EventType::CallReject:
+        return "m.call.reject";
+    case EventType::CallNegotiate:
+        return "m.call.negotiate";
     case EventType::SecretRequest:
         return "m.secret.request";
     case EventType::SecretSend:
