@@ -984,6 +984,31 @@ TEST(ClientAPI, Versions)
     mtx_client->close();
 }
 
+TEST(ClientAPI, Capabilities)
+{
+    std::shared_ptr<Client> mtx_client = make_test_client();
+
+    mtx_client->login(
+      "alice", "secret", [mtx_client](const mtx::responses::Login &, RequestErr err) {
+          check_error(err);
+          mtx_client->capabilities(
+            [](const mtx::responses::capabilities::Capabilities &res, RequestErr err) {
+                check_error(err);
+
+                EXPECT_GE(res.room_versions.default_.size(), 1);
+                EXPECT_GE(res.room_versions.available.size(), 1);
+                EXPECT_EQ(res.room_versions.available.at(res.room_versions.default_),
+                          mtx::responses::capabilities::RoomVersionStability::Stable);
+                EXPECT_EQ(res.change_3pid.enabled, true);
+                EXPECT_EQ(res.change_password.enabled, true);
+                EXPECT_EQ(res.set_avatar_url.enabled, true);
+                EXPECT_EQ(res.set_displayname.enabled, true);
+            });
+      });
+
+    mtx_client->close();
+}
+
 TEST(ClientAPI, Typing)
 {
     auto alice = make_test_client();
@@ -1008,7 +1033,8 @@ TEST(ClientAPI, Typing)
             alice->sync(opts, [room_id, alice](const mtx::responses::Sync &res, RequestErr err) {
                 check_error(err);
 
-                auto room = res.rooms.join.at(room_id);
+                auto room       = res.rooms.join.at(room_id);
+                auto next_batch = res.next_batch;
 
                 EXPECT_EQ(room.ephemeral.events.size(), 1);
                 EXPECT_EQ(std::get<mtx::events::EphemeralEvent<mtx::events::ephemeral::Typing>>(
@@ -1016,11 +1042,12 @@ TEST(ClientAPI, Typing)
                             .content.user_ids.front(),
                           "@alice:" + server_name());
 
-                alice->stop_typing(room_id, [alice, room_id](RequestErr err) {
+                alice->stop_typing(room_id, [alice, room_id, next_batch](RequestErr err) {
                     check_error(err);
 
                     SyncOpts opts;
                     opts.timeout = 0;
+                    opts.since   = next_batch;
                     alice->sync(opts, [room_id](const mtx::responses::Sync &res, RequestErr err) {
                         check_error(err);
                         auto room = res.rooms.join.at(room_id);
@@ -1090,7 +1117,7 @@ TEST(ClientAPI, PresenceOverSync)
     bob->login(
       "bob", "secret", [](const mtx::responses::Login &, RequestErr err) { check_error(err); });
 
-    while (alice->access_token().empty() && bob->access_token().empty())
+    while (alice->access_token().empty() || bob->access_token().empty())
         sleep();
 
     std::atomic<bool> can_exit = false;
@@ -1408,7 +1435,9 @@ TEST(ClientAPI, Pagination)
                   check_error(err);
 
                   // We reached the start of the timeline.
-                  EXPECT_EQ(res.start, res.end);
+                  // Old synapse versions send start == end in that case, newer ones send an empty
+                  // token.
+                  EXPECT_TRUE(res.start == res.end || res.end.empty());
                   EXPECT_EQ(res.chunk.size(), 0);
               });
           });
@@ -1815,3 +1844,47 @@ TEST(ClientAPI, PublicRooms)
     alice->close();
     bob->close();
 }
+
+TEST(ClientAPI, Summary)
+{
+    // Setup : Create a new (public) room with some settings
+    auto alice = make_test_client();
+    auto bob   = make_test_client();
+
+    alice->login("alice", "secret", [alice](const mtx::responses::Login &, RequestErr err) {
+        check_error(err);
+    });
+
+    bob->login(
+      "bob", "secret", [bob](const mtx::responses::Login &, RequestErr err) { check_error(err); });
+
+    while (alice->access_token().empty() || bob->access_token().empty())
+        sleep();
+
+    std::string room_name = "Public Room" + alice->generate_txn_id();
+    mtx::requests::CreateRoom req;
+    req.name            = room_name;
+    req.topic           = "Test";
+    req.visibility      = mtx::common::RoomVisibility::Public;
+    req.invite          = {"@bob:" + server_name()};
+    req.room_alias_name = alice->generate_txn_id();
+    req.preset          = Preset::PublicChat;
+
+    alice->create_room(
+      req, [alice, bob, room_name](const mtx::responses::CreateRoom &res, RequestErr err) {
+          check_error(err);
+          auto room_id = res.room_id;
+
+          bob->get_summary(room_id.to_string(),
+                           [room_name](const mtx::responses::PublicRoom &res, RequestErr err) {
+                               check_error(err);
+                               EXPECT_EQ(res.name, room_name);
+                               EXPECT_NE(res.room_version, "1");
+                               EXPECT_NE(res.room_version, "");
+                               EXPECT_EQ(res.membership, mtx::events::state::Membership::Invite);
+                           });
+      });
+
+    alice->close();
+    bob->close();
+}
